import http from "@/api/http.js"


/* 数据报表模块 */
const dataMainInterface = {
	/* 工作台 */
	workdata: params => {
		return http.get('/statistic_report/home')
	},
	/**
	 * 获取即将过期的客户列表接口
	 * @param {PageSize } params 
	 * @param {CurrentIndex } params 
	 * @param {EndDate } params 结束日期
	 * @param {CompanyType} params 
	 * @param {AdminId } params 销售id，多个用英文逗号隔开，空字符串为全部
	 * @param {RegionType } params 
	 * @returns 
	 */
	expireList:params => {
		return http.get('/statistic_report/will_expire_list',params);
	},
	/**
	 * 获取存量客户列表接口
	 * @param {PageSize } params 
	 * @param {CurrentIndex } params 
	 * @param {CompanyType} params 
	 * @param {AdminId } params 销售id，多个用英文逗号隔开，空字符串为全部
	 * @param {RegionType } params 
	 * @param {Date} params 
	 * @param {DataType} params 
	 * @returns 
	 */
	stackList:params => {
		return http.get('/statistic_report/stack_company_list',params);
	},
	/**
	 * 获取增量客户列表接口
	 * @param {PageSize} params 
	 * @param {CurrentIndex} params 
	 * @param {EndDate} params 结束日期
	 * @param {CompanyType} params 
	 * @param {AdminId } params 销售id，多个用英文逗号隔开，空字符串为全部
	 * @param {RegionType } params 
	 * @param {StartDate} params 
	 * @param {DataType} params 
	 * @returns 
	 */
	incrementalList:params => {
		return http.get('/statistic_report/incremental_company_list',params);
	},
	/**
	 * 获取收入统计列表接口
	 * @param {PageSize} params 
	 * @param {CurrentIndex} params 
	 * @param {EndDate} params 结束日期
	 * @param {CompanyType} params 
	 * @param {AdminId } params 销售id，多个用英文逗号隔开，空字符串为全部
	 * @param {RegionType } params
	 * @returns 
	 */
	incomeList:params => {
		return http.get('/statistic_report/income_list',params);
	},
	/**
	 * 获取合同数据报表接口
	 * @param {PageSize} params 
	 * @param {CurrentIndex} params 
	 * @param {EndDate} params 结束日期
	 * @param {CompanyType} params 
	 * @param {AdminId } params 销售id，多个用英文逗号隔开，空字符串为全部
	 * @param {RegionType } params
	 * @returns 
	 */
	contractList:params => {
		return http.get('/statistic_report/contract_list',params);
	},
	/**
	 * 获取企业客户的合同数据报表接口
	 * @param {CompanyId  } params
	 * @returns 
	 */
	companyContractList:params => {
		return http.get('/statistic_report/company_contract_list',params);
	},
	// 系统合同类调此接口
	companyContractDetailList:params => {
		return http.get('/statistic_report/contract_detail_list',params);
	},

	// 客户数据未续约说明更多
	moreRenewReason:params => {
		return http.get('/statistic_report/more_renew_reason',params);
	},
	/**
	 * 	// 获取图表阅读统计列表
	 * @param {PageSize} params 
	 * @param {CurrentIndex} params 
	 * @param {ClassifyId} params 分类ID
	 * @param {Order} params 排序
	 * @returns 
	 */
	getchartCensusList:params => {
		return http.get('/yb/chartCensus/getVisitList',params);
	},
	/**
	 * 	// 图表阅读统计详情列表
	 * @param {PageSize} params 
	 * @param {CurrentIndex} params 
	 * @param {ClassifyId} params 分类ID
	 * @param {Order} params 排序
	 * @returns 
	 */
	 getchartgetVisitDetailList:params => {
	   return http.get('yb/chartCensus/getVisitDetailList',params);
	 },
	 /**
	 * 	// 用户图表访问明细
	 * @returns 
	 */
	  getCompanyChartVisitDetail:params => {
		return http.get('yb/chartCensus/getCompanyChartVisitDetail',params);
	  },
	 /**
	 * 	// 图库权限开通客户统计列表
	 * @returns 
	 */
	 getCompanyAuthCensusList:params => {
		return http.get('yb/chartCensus/getCompanyAuthCensusList',params);
	  },
	  
	/**新增客户统计 
	 * StartDate EndDate DataType
	 * @return 
	 */
	newcustomStatistic: params => {
		return http.get('/statistic_report/report/incremental_company',params)
	},
	/**
	 * 存量客户统计
	 * Date
	 * @return
	 */
	stockcustomStatistic: params => {
		return http.get('/statistic_report/report/stack_company',params)
	},
	/**续约客户统计 
	 * StartDate 统计开始时间
	 * EndDate 统计结束时间
	 * DataType 统计范围类型
	 * ProductId 项目类型 1 ficc  2 权益
	 * @return 
	 */
	 renewalCustomStatistic: params => {
		return http.get('/statistic_report/report/renew_company',params)
	},

	/**
	 * 新增客户列表
	 * @param {} params PageSize CurrentIndex SortParam SortType 	CompanyIds
	 * @return
	 */
	newcustomList: params => {
		return http.post('/statistic_report/report/company_list',params)
	},
	/**
	 * 销售To do统计
	 * DataType ProductId StartDate EndDate
	 * @return
	 */
	 todoStatistic: params => {
		return http.get('/statistic_report/report/seller_company_todo',params)
	},

	/**
	 * 语音播报统计
	 * @param SectionId	板块id
	 * @param VarietyId 品种id
	 * @param AdminId  语音管理员id
	 * @param ClickSort 排序 1升序 2降序
	 * @param Title 搜索关键词
	 */
	voicePlayStatistic:params=>{
		return http.get('/yb/voice/broadcast/list',params)
	},

	/**
	 * 语音播报详情
	 * @param BroadcastId
	 */
	voicePlayDetailSta:params=>{
		return http.get('/yb/voice/broadcast/deatil',params)
	},

	/**
	 * 价格驱动点击量-列表
	 */
	priceDrivenStatisticsList:params=>{
		return http.get('/yb/price_driven/click_census_list',params)
	},
	/**
	 * 价格驱动点击量-详情
	 */
	 priceDrivenStatisticsDetail:params=>{
		return http.get('/yb/price_driven/click_census_detail',params)
	},

	//ficc产品阅读统计
	ficcProductStatistic:params=>{
		return http.get('/statistic_report/report/seller_yb_log',params)
	},

	/**
	 * 分产品阅读客户统计-- 新增客户列表
	 * @param {} params PageSize CurrentIndex SortParam SortType SellerIds ProductId ProductType StartDate EndDate CompanyStatus
	 * @return
	*/
		newCustomProductionList: params => {
		return http.post('/statistic_report/report/seller_yb_log/user',params)
	},

  /**
   * 分产品阅读统计 -- 收藏统计
   * @param Datatype
   */
  collectStatistic:params=>{
    return http.get('/statistic_report/report/seller_collect_log',params)
  },

  /**
   * 分产品阅读统计 -- 收藏统计用户详情
   * @param Datatype
   */
   collectStatisticList:params=>{
    return http.post('/statistic_report/report/seller_collect_log/user',params)
  },

  /**
   * 分产品阅读统计 -- 收藏详情详情
   * @param CollectionType
   */
   collectDetailList:params=>{
    return http.post('/statistic_report/report/seller_collect_log/detail',params)
  },
   /**
   * 分产品阅读统计 -- 图收藏统计
   * @param Datatype
   */
    chartStatistic:params=>{
      return http.get('/statistic_report/report/seller_chart_collect_log',params)
    },
  /**
   * 分产品阅读统计 -- 图收藏统计用户详情
   * @param Datatype
   */
  chartStatisticList:params=>{
    return http.post('/statistic_report/report/seller_chart_collect_log/user',params)
  },
  /**
   * 分产品阅读统计 -- 图收藏详情列表
   * @param CollectionType
   */
   chartDetailList:params=>{
    return http.post('/statistic_report/report/seller_chart_collect_log/detail',params)
  },

}

export {
	dataMainInterface
}