import { Graph, Cell, Node, Path } from '@antv/x6'
import Hierarchy from '@antv/hierarchy'

export default {
  data() {
    return {
      mindMapDataCurrent:{},
      positionCurrent:{},
      addTypeCurrent:'',
      styleConfig:{}
    }
  },
  mounted() {
    
  },
  created() {
    this.styleConfig=this.$store.state.sand.styleConfig
    // console.log(this.styleConfig,'this.styleConfig');
    // 中心主题
    Graph.registerNode(
      'mindmap-topic',          
      {
        inherit: 'rect',
        markup: [
          {
            tagName: 'rect',
            selector: 'body',
          },
          {
            tagName: 'image',
            selector: 'leftImg',
          },
          {
            tagName: 'image',
            selector: 'rightImg',
          },
          {
            tagName: 'text',
            selector: 'text',
          },
        ],
        attrs: {
          body: {
            rx: 6,
            ry: 6,
            // stroke: this.$store.state.sand.styleConfig.borderColor,
            // fill: this.$store.state.sand.styleConfig.backgroundColor,
            strokeWidth: 1,
            width:100,
            height:50
          },
          leftImg: {
            ref: 'body',
            refX: -16,
            refY: '50%',
            refY2: -8,
            width: 16,
            height: 16,
            'xlink:href':
              'https://gw.alipayobjects.com/mdn/rms_43231b/afts/img/A*SYCuQ6HHs5cAAAAAAAAAAAAAARQnAQ',
            event: 'add:topic:left',
            class: 'left-topic-image',
          },
          rightImg: {
            ref: 'body',
            refX: '100%',
            refY: '50%',
            refY2: -8,
            width: 16,
            height: 16,
            'xlink:href':
              'https://gw.alipayobjects.com/mdn/rms_43231b/afts/img/A*SYCuQ6HHs5cAAAAAAAAAAAAAARQnAQ',
            event: 'add:topic:right',
            class: 'right-topic-image',
          },
          text: {
            fontSize: 14,
            fill: this.$store.state.sand.styleConfig.color,
            textWrap:{
              width:-10
            }
          }
        },
      },
      true,
    )
    // 左分支主题
    Graph.registerNode(
      'mindmap-topic-left',          
      {
        inherit: 'rect',
        markup: [
          {
            tagName: 'rect',
            selector: 'body',
          },
          {
            tagName: 'image',
            selector: 'leftImg',
          },
          {
            tagName: 'image',
            selector: 'rightImg',
          },
          {
            tagName: 'text',
            selector: 'text',
          },
        ],
        attrs: {
          body: {
            rx: 6,
            ry: 6,
            // stroke: this.$store.state.sand.styleConfig.borderColor,
            // fill: this.$store.state.sand.styleConfig.backgroundColor,
            strokeWidth: 1,
            width:100,
            height:50
          },
          leftImg: {
            ref: 'body',
            refX: -16,
            refY: '50%',
            refY2: -8,
            width: 16,
            height: 16,
            'xlink:href':
              'https://gw.alipayobjects.com/mdn/rms_43231b/afts/img/A*SYCuQ6HHs5cAAAAAAAAAAAAAARQnAQ',
            event: 'add:topic:left',
            class: 'left-topic-image',
          },
          text: {
            fontSize: 14,
            fill: this.$store.state.sand.styleConfig.color,
            textWrap:{
              width:-10
            }
          }
        },
      },
      true,
    )    
    // 右分支主题
    Graph.registerNode(
      'mindmap-topic-right',          
      {
        inherit: 'rect',
        markup: [
          {
            tagName: 'rect',
            selector: 'body',
          },
          {
            tagName: 'image',
            selector: 'leftImg',
          },
          {
            tagName: 'image',
            selector: 'rightImg',
          },
          {
            tagName: 'text',
            selector: 'text',
          },
        ],
        attrs: {
          body: {
            rx: 6,
            ry: 6,
            // stroke: this.$store.state.sand.styleConfig.borderColor,
            // fill: this.$store.state.sand.styleConfig.backgroundColor,
            strokeWidth: 1,
            width:100,
            height:50
          },
          rightImg: {
            ref: 'body',
            refX: '100%',
            refY: '50%',
            refY2: -8,
            width: 16,
            height: 16,
            'xlink:href':
              'https://gw.alipayobjects.com/mdn/rms_43231b/afts/img/A*SYCuQ6HHs5cAAAAAAAAAAAAAARQnAQ',
            event: 'add:topic:right',
            class: 'right-topic-image',
          },
          text: {
            fontSize: 14,
            fill:this.$store.state.sand.styleConfig.color,
            textWrap:{
              width:-10
            }
          }
        },
      },
      true,
    )
    // 分支主题
    // Graph.registerNode(
    //   'mindmap-topic-child',
    //   {
    //     inherit: 'rect',
    //     markup: [
    //       {
    //         tagName: 'rect',
    //         selector: 'body',
    //       },
    //       {
    //         tagName: 'text',
    //         selector: 'text',
    //       },
    //       {
    //         tagName: 'path',
    //         selector: 'line',
    //       },
    //     ],
    //     attrs: {
    //       body: {
    //         fill: '#ffffff',
    //         strokeWidth: 0,
    //         stroke: '#5F95FF',
    //       },
    //       text: {
    //         fontSize: 14,
    //         fill: '#262626',
    //         textVerticalAnchor: 'bottom',
    //       },
    //       line: {
    //         stroke: '#5F95FF',
    //         strokeWidth: 2,
    //         d: 'M 0 15 L 60 15',
    //       },
    //     },
    //   },
    //   true,
    // )

    // 连接器
    Graph.registerConnector(
      'mindmap',
      (sourcePoint, targetPoint, routerPoints, options) => {
        // console.log(sourcePoint, targetPoint, routerPoints, options,'sourcePoint, targetPoint, routerPoints, options');
        const midX = sourcePoint.x + 10
        const midY = sourcePoint.y
        const ctrX = (targetPoint.x - midX) / 5 + midX
        const ctrY = targetPoint.y
        const pathData = `
        M ${sourcePoint.x} ${sourcePoint.y}
        Q ${ctrX} ${ctrY} ${targetPoint.x} ${targetPoint.y}
        `
        return options.raw ? Path.parse(pathData) : pathData
      },
      true,
    )

    // 边
    Graph.registerEdge(
      'mindmap-edge',
      {
        inherit: 'edge',
        connector: {
          name: 'mindmap',
        },
        attrs: {
          line: {
            targetMarker: false,
            // stroke: this.$store.state.sand.styleConfig.lineColor,
            strokeWidth: 2,
          },
        },
        zIndex: 0,
      },
      true,
    )
  },
  methods: {
    setGraph(){
      this.graph.on('add:topic:left', ({ node }) => {
        if(this.operationType=='view') return 
        const { id } = node
        this.setCurrent(id)
        const type = node.prop('type')
        if (this.addChildNode(id, type,'left')) {
          this.mindMapRender()      
        }
      })

      this.graph.on('add:topic:right', ({ node }) => {
        if(this.operationType=='view') return 
        const { id } = node
        this.setCurrent(id)
        const type = node.prop('type')
        if (this.addChildNode(id, type,'right')) {
          this.mindMapRender()      
        }
      })
      this.graph.on('node:change:size', (args) => { 
        if(this.operationType=='view') return 
        if(args.node.shape.indexOf('mindmap')!==-1){
          let ids = args.node.id.split('-')
					let mindmapDataIndex = this.mindmapDataUse.findIndex(mindmap => mindmap.mindmapData.id == ids[0])
					let mindMapDataCurrent = this.mindmapDataUse[mindmapDataIndex]?this.mindmapDataUse[mindmapDataIndex].mindmapData:{}

					let findId = ids[0]

					for (let i = 1; i < ids.length; i++) {
						const element = ids[i];
						findId = findId+'-'+element
						mindMapDataCurrent=mindMapDataCurrent.children.find(it => it.id==findId)
					}

					mindMapDataCurrent.width = args.node.size().width
					mindMapDataCurrent.height = args.node.size().height
        }
      })
      this.graph.on('node:change:position', (args) => { 
        if(this.operationType=='view') return 
        if(args.node.shape.indexOf('mindmap')!==-1 && Number(args.node.id)){

          let index = this.mindmapDataUse.findIndex(it =>it.mindmapData.id == args.node.id)
          // let index = Number(args.node.id)
          if(index!=-1){
            this.mindmapDataUse[index].position = args.current
          // this.setCurrent(args.node.id)
          }
        }
      })
      // this.graph.on('node:added', ({node}) => { 
      //   if(node.shape.indexOf('mindmap')!=-1){
      //     // 思维导图需要处理相对应的数据
      //     console.log('mindmap',node.id);
      //   }
      //   console.log(node,'node:added');
      // })
      // this.graph.on('node:removed', (args) => { 
      //   console.log(args,'node:removed');
      // })
      // this.graph.bindKey(['backspace', 'delete'], () => {
      //   const selectedNodes = this.graph.getSelectedCells().filter((item) => item.isNode())
      //   if (selectedNodes.length) {
      //     const { id } = selectedNodes[0]
      //     if (this.removeNode(id)) {
      //       this.mindMapRender()
      //     }
      //   }
      // })
      
      this.graph.bindKey('tab', (e) => {  
        if(this.operationType=='view') return 
        e.preventDefault()
        const selectedNodes = this.graph.getSelectedCells().filter((item) => {
          return item.shape.indexOf('mindmap')!=-1 && item.isNode()
        })
        if (selectedNodes.length) {
          const node = selectedNodes[0]
          this.setCurrent(node.id)
          let type = node.prop('type')
          let direction = node.shape.indexOf('left')!=-1?'left':'right'
          if (this.addChildNode(node.id, type,direction)) {
            this.mindMapRender()
          }
        }
      })
    },
    // 通过id设置当前操作的思维导图
    setCurrent(id){
      let rootId = id.split('-')[0]
      let index = this.mindmapDataUse.findIndex(it =>it.mindmapData.id == rootId)
      this.mindMapDataCurrent = this.mindmapDataUse[index]?this.mindmapDataUse[index].mindmapData:{}
      this.positionCurrent = this.mindmapDataUse[index]?this.mindmapDataUse[index].position:{x:0,y:0}
      this.addTypeCurrent = this.mindmapDataUse[index]?this.mindmapDataUse[index].addType:'singleMindmap'
    },
    mindMapRender(i){
      this.graph.startBatch('renderMindmap')
      let mindMapType = i || i==0?this.mindmapDataUse[i].addType:this.addTypeCurrent
      this.mindMapDataCurrent = i || i==0?this.mindmapDataUse[i].mindmapData:this.mindMapDataCurrent
      this.positionCurrent = i || i==0?this.mindmapDataUse[i].position:this.positionCurrent
      const result = Hierarchy.mindmap(this.mindMapDataCurrent, {
        direction: 'H',
        getHeight(d) {
          return d.height
        },
        getWidth(d) {
          return d.width
        },
        getHGap() {
          return 40
        },
        getVGap() {
          return 20
        },
        getSide: (d) => {
          return mindMapType.indexOf('double') != -1?d.data.direction || 'left':'right'
        }
      })
      const cells = []
      let xGap = this.positionCurrent?this.positionCurrent.x-result.x:0
      let yGap = this.positionCurrent?this.positionCurrent.y-result.y:0
      // console.log(result,'result');
      // return 
      const traverse = (hierarchyItem) => {
        if (hierarchyItem) {
          const { data, children } = hierarchyItem
          // console.log(hierarchyItem,'hierarchyItemhierarchyItemhierarchyItem');
          let mindmapDirection = mindMapType.indexOf('double') != -1?data.direction:'right'
          // console.log(mindmapDirection,data.label,data.direction);
          let currentCell=this.graph.getCellById(data.id)

          if(!currentCell){
            // 没有 新增
            cells.push(
              this.graph.createNode({
                id: data.id,
                shape:mindmapDirection=='right'?'mindmap-topic-right':mindmapDirection=='left'?'mindmap-topic-left':'mindmap-topic',
                x: xGap+hierarchyItem.x,
                y: yGap+hierarchyItem.y,
                width: data.width,
                height: data.height,
                label: data.label,
                type: data.type,
                attrs:{
                  body: {
                    stroke: this.$store.state.sand.styleConfig.borderColor,
                    fill: this.$store.state.sand.styleConfig.backgroundColor
                  },
                  text:{
                    fill:this.$store.state.sand.styleConfig.color
                  }
                },
              }),
            )
          }else{
            // 有，更新下位置信息
            currentCell.position(xGap+hierarchyItem.x,yGap+hierarchyItem.y)
          }
          // return 
   
          if (children) {
            children.forEach((item) => {
              const { id, data } = item
              let mindmapChildDirection = mindMapType.indexOf('double') != -1?data.direction:'right'
              let currentEdge=this.graph.getCellById(data.id)
              if(!currentEdge){
                cells.push(
                  this.graph.createEdge({
                    shape: 'mindmap-edge',
                    attrs:{
                      line:{
                        stroke:this.$store.state.sand.styleConfig.lineColor
                      }
                    },
                    source: {
                      cell: hierarchyItem.id,
                      anchor: {
                        name: 'center',
                        args: {
                          dx: mindmapChildDirection=='right'?'25%':mindmapChildDirection=='left'?'-25%':0,
                        },
                      },
                    },
                    target: {
                      cell: id,
                      anchor: {
                        name: mindmapChildDirection =='left'?'right':'left',
                      },
                    },
                  }),
                )
              }

              traverse(item)
            })
          }
        }
      }
      traverse(result)
      // 排下序，把边放最后面 不然 边 会找不到 节点
      let sortCells = cells.sort(cell => cell.shape.indexOf('edge'))
      // console.log(sortCells,'result');
      // return 
      // this.graph.removeCells(sortCells)
      this.graph.addCell(sortCells)
      this.graph.stopBatch('renderMindmap')
      // this.graph.resetCells(cells)
      // this.graph.centerContent()
    },
    findItem(obj,id){
      if (obj.id === id) {
        return {
          parent: null,
          node: obj,
        }
      }
      const { children } = obj
      if (children) {
        for (let i = 0, len = children.length; i < len; i++) {
          const res = this.findItem(children[i], id)
          if (res) {
            return {
              parent: res.parent || obj,
              node: res.node,
            }
          }
        }
      }
      return null
    },
    addChildNode (id, type,direction='left'){
      console.log('添加');
      // 重做不了 清空重做栈
      this.mindmapAssistData.mindmapDataRecoverUse = []
      // console.log(this.mindmapAssistData);

      const res = this.findItem(this.mindMapDataCurrent, id)
      const dataItem = res && res.node
      if (dataItem) {
        let item = null
        // console.log(dataItem.children);
        let addId ='1'
        if(dataItem.children && dataItem.children.length>0){
          let ids = dataItem.children[dataItem.children.length-1].id.split('-')
          addId = parseInt(ids[ids.length-1])+1+''
        }
        if (type === 'topic') {
          item = {
            id: `${id}-${addId}`,
            type: 'topic-branch',
            label: `分支主题${addId}`,
            width: 100,
            height: 40,
            direction
          }
        } else if (type === 'topic-branch' || type=='topic-child') {
          item = {
            id: `${id}-${addId}`,
            type: 'topic-child',
            label: `子主题${addId}`,
            width: 60,
            height: 30,
            direction
          }
        }
        if (item) {
          if (dataItem.children) {
            dataItem.children.push(item)
          } else {
            dataItem.children = [item]
          }
          return item
        }
      }
      return null
    },
    removeNode (id) {
      const res = this.findItem(this.mindMapDataCurrent, id)
      const dataItem = res && res.parent
      if (dataItem && dataItem.children) {
        const { children } = dataItem
        const index = children.findIndex((item) => item.id === id)
        return children.splice(index, 1)
      }
      return null
    }
  },
}
